@@ -23,24 +23,33 @@ import "C"
 import (
 	"context"
 	"flag"
+	"fmt"
+	"net"
 	"os"
+	"os/exec"
 	"os/signal"
+	"path/filepath"
+	"strconv"
+	"strings"
 	"sync"
 	"syscall"
 	"time"
 
 	dbus "github.com/godbus/dbus"
+	accounts "github.com/linuxdeepin/go-dbus-factory/com.deepin.daemon.accounts"
+	notifications "github.com/linuxdeepin/go-dbus-factory/org.freedesktop.notifications"
 	x "github.com/linuxdeepin/go-x11-client"
-	"pkg.deepin.io/dde/startdde/display"
-	"pkg.deepin.io/dde/startdde/iowait"
-	"pkg.deepin.io/dde/startdde/watchdog"
-	wl_display "pkg.deepin.io/dde/startdde/wl_display"
-	"pkg.deepin.io/dde/startdde/wm_kwin"
-	"pkg.deepin.io/dde/startdde/xsettings"
-	"pkg.deepin.io/lib/dbusutil"
-	"pkg.deepin.io/lib/gsettings"
-	"pkg.deepin.io/lib/log"
-	"pkg.deepin.io/lib/proxy"
+	"github.com/linuxdeepin/startdde/display"
+	"github.com/linuxdeepin/startdde/iowait"
+	"github.com/linuxdeepin/startdde/watchdog"
+	wl_display "github.com/linuxdeepin/startdde/wl_display"
+	"github.com/linuxdeepin/startdde/wm_kwin"
+	"github.com/linuxdeepin/startdde/xsettings"
+	"github.com/linuxdeepin/go-lib/dbusutil"
+	"github.com/linuxdeepin/go-lib/gettext"
+	"github.com/linuxdeepin/go-lib/gsettings"
+	"github.com/linuxdeepin/go-lib/log"
+	"github.com/linuxdeepin/go-lib/proxy"
 )
 
 var logger = log.NewLogger("startdde")
@@ -89,10 +98,17 @@ func shouldUseDDEKWin() bool {
 }
 
 const (
-	cmdKWin             = "/usr/bin/kwin_no_scale"
-	cmdDdeSessionDaemon = "/usr/lib/deepin-daemon/dde-session-daemon"
-	cmdDdeDock          = "/usr/bin/dde-dock"
-	cmdDdeDesktop       = "/usr/bin/dde-desktop"
+	cmdKWin                = "/usr/bin/kwin_no_scale"
+	cmdDdeSessionDaemon    = "/usr/lib/deepin-daemon/dde-session-daemon"
+	cmdDdeDock             = "/usr/bin/dde-dock"
+	cmdDdeDesktop          = "/usr/bin/dde-desktop"
+	cmdLoginReminderHelper = "/usr/libexec/deepin/login-reminder-helper"
+	cmdDdeHintsDialog      = "/usr/bin/dde-hints-dialog"
+
+	loginReminderTimeout    = 5 * time.Second
+	loginReminderTimeFormat = "2006-01-02 15:04:05"
+	secondsPerDay           = 60 * 60 * 24
+	accountUserPath         = "/com/deepin/daemon/Accounts/User"
 )
 
 func launchCoreComponents(sm *SessionManager) {
@@ -134,13 +150,41 @@ func launchCoreComponents(sm *SessionManager) {
 				handleKWinReady(sm)
 			})
 		} else {
-			initGSettingsConfig()
 			wmCmd := _gSettingsConfig.wmCmd
 			if wmCmd == "" {
 				wmCmd = "x-window-manager"
 			}
 			launch("env", []string{"GDK_SCALE=1", wmCmd}, "wm", false, nil)
 		}
+	} else {
+		var handleId dbusutil.SignalHandlerId
+		var needDel = true
+		handleId, err := sm.dbusDaemon.ConnectNameOwnerChanged(func(name, newOwner, oldOwner string) {
+			if name == "org.kde.KWin" && newOwner != "" && oldOwner == "" {
+				handleKWinReady(sm)
+				sm.dbusDaemon.RemoveHandler(handleId)
+			}
+		})
+		if err != nil {
+			logger.Warningf("connect to signal err: %s, run handle after timeout", err)
+			time.AfterFunc(time.Second*5, func() {
+				handleKWinReady(sm)
+			})
+			needDel = false
+		}
+
+		hasOwner, err := sm.dbusDaemon.NameHasOwner(0, "org.kde.KWin")
+		if err != nil {
+			logger.Warning(err)
+			return
+		}
+
+		if hasOwner {
+			handleKWinReady(sm)
+			if needDel {
+				sm.dbusDaemon.RemoveHandler(handleId)
+			}
+		}
 	}
 
 	// 先启动 dde-session-daemon，再启动 dde-dock
@@ -149,7 +193,7 @@ func launchCoreComponents(sm *SessionManager) {
 		if _useKWin {
 			dockArgs = []string{"-r"}
 		}
-		launch(cmdDdeDock, dockArgs, "dde-dock", true, nil)
+		launch(cmdDdeDock, dockArgs, "dde-dock", _useKWin, nil)
 	})
 	launch(cmdDdeDesktop, nil, "dde-desktop", true, nil)
 
@@ -200,9 +244,48 @@ func logInfoAfter(msg string) {
 	logger.Infof("after %s, %s", elapsed, msg)
 }
 
+func greeterDisplayMain() {
+	display.SetGreeterMode(true)
+	// init x conn
+	xConn, err := x.NewConn()
+	if err != nil {
+		logger.Warning(err)
+		os.Exit(1)
+	}
+	// TODO
+	display.Init(xConn, false)
+	logger.Debug("greeter mode")
+	service, err := dbusutil.NewSessionService()
+	if err != nil {
+		logger.Warning(err)
+	}
+	err = display.Start(service)
+	if err != nil {
+		logger.Warning(err)
+	}
+	err = display.StartPart2()
+	if err != nil {
+		logger.Warning(err)
+	}
+	service.Wait()
+}
+
 func main() {
-	_mainBeginTime = time.Now()
 	flag.Parse()
+	if len(os.Args) > 0 && strings.HasPrefix(filepath.Base(os.Args[0]), "greeter") {
+		// os.Args[0] 应该一般是 greeter-display-daemon
+		greeterDisplayMain()
+		return
+	}
+
+	initGSettingsConfig()
+
+	_mainBeginTime = time.Now()
+
+	gettext.InitI18n()
+	gettext.BindTextdomainCodeset("startdde", "UTF-8")
+	gettext.Textdomain("startdde")
+
 	reapZombies()
 	// init x conn
 	xConn, err := x.NewConn()
@@ -222,16 +305,10 @@ func main() {
 	if os.Getenv("WAYLAND_DISPLAY") != "" {
 		logger.Info("in wayland mode")
 		_useWayland = true
-		// 相比于 X11 环境，在 Wayland 环境下，先于启动核心组件之前启动了 wl_display 模块。
-		err := wl_display.Start()
-		if err != nil {
-			logger.Warning(err)
-		}
-		recommendedScaleFactor = wl_display.GetRecommendedScaleFactor()
-	} else {
-		display.Init(xConn)
-		recommendedScaleFactor = display.GetRecommendedScaleFactor()
 	}
+	display.Init(xConn, _useWayland)
+	// TODO
+	recommendedScaleFactor = display.GetRecommendedScaleFactor()
 
 	service, err := dbusutil.NewSessionService()
 	if err != nil {
@@ -239,7 +316,7 @@ func main() {
 		os.Exit(1)
 	}
 
-	xsManager, err := xsettings.Start(xConn, logger, recommendedScaleFactor, service)
+	xsManager, err := xsettings.Start(xConn, recommendedScaleFactor, service, &display.ScaleFactorsHelper)
 	if err != nil {
 		logger.Warning(err)
 	} else {
@@ -257,28 +334,21 @@ func main() {
 	}
 	logDebugAfter("before launchCoreComponents")
 
-	if !_useWayland {
-		// 使用 X11 环境时, 把 display 模块的启动分成两个部分，前一部分在 core components 启动之前启动，
-		// 后一部分在 core components 启动之后启动。
-		err := display.Start(service)
-		if err != nil {
-			logger.Warning("start display part1 failed:", err)
-		}
+	err = display.Start(service)
+	if err != nil {
+		logger.Warning("start display part1 failed:", err)
 	}
 
 	launchCoreComponents(sessionManager)
 
-	if !_useWayland {
-		// 启动 display 模块的后一部分
-		go func() {
-			err := display.StartPart2()
-			if err != nil {
-				logger.Warning("start display part2 failed:", err)
-			}
-		}()
-	}
+	// 启动 display 模块的后一部分
+	go func() {
+		err := display.StartPart2()
+		if err != nil {
+			logger.Warning("start display part2 failed:", err)
+		}
+	}()
 
-	initGSettingsConfig()
 	go func() {
 		initSoundThemePlayer()
 		playLoginSound()
@@ -315,6 +385,9 @@ func main() {
 	}
 
 	go handleOSSignal(sessionManager)
+
+	loginReminder()
+
 	service.Wait()
 }
 
@@ -344,3 +417,137 @@ func doSetLogLevel(level log.Priority) {
 	}
 	watchdog.SetLogLevel(level)
 }
+
+func loginReminder() {
+	if !_gSettingsConfig.loginReminder {
+		return
+	}
+
+	sysBus, _ := dbus.SystemBus()
+
+	uid := os.Getuid()
+	userPath := accountUserPath + strconv.Itoa(uid)
+
+	user, err := accounts.NewUser(sysBus, dbus.ObjectPath(userPath))
+	if err != nil {
+		logger.Warning("failed to get user:", err)
+	}
+
+	res, err := user.GetReminderInfo(0)
+	if err != nil {
+		logger.Warning("failed to get reminder info:", err)
+	}
+
+	currentLoginTime, err := time.Parse("2006-01-02 15:04:05.999999999 -0700 MST", res.CurrentLogin.Time)
+	if err != nil {
+		logger.Warning("wrong time format:", err)
+	}
+	currentLoginTimeStr := currentLoginTime.Format(loginReminderTimeFormat)
+
+	address := res.CurrentLogin.Address
+	if address == "0.0.0.0" {
+		// 若 IP 是「0.0.0.0」，获取当前设备的 IP
+		address = getFirstIPAddress()
+	}
+
+	body := fmt.Sprintf("%s %s %s", res.Username, address, currentLoginTimeStr)
+
+	// pam_unix/passverify.c
+	curDays := int(time.Now().Unix() / secondsPerDay)
+	daysLeft := res.Spent.LastChange + res.Spent.Max - curDays
+	if res.Spent.Max != -1 && res.Spent.Warn != -1 {
+		if res.Spent.Warn > daysLeft {
+			body += " " + fmt.Sprintf(gettext.Tr("Your password will expire in %d days"), daysLeft)
+		}
+	}
+
+	body += "\n" + fmt.Sprintf(gettext.Tr("%d login failures since the last successful login"), res.FailCountSinceLastLogin)
+
+	bus, _ := dbus.SessionBus()
+	notifi := notifications.NewNotifications(bus)
+	sigLoop := dbusutil.NewSignalLoop(bus, 10)
+	sigLoop.Start()
+	notifi.InitSignalExt(sigLoop, true)
+
+	// TODO: icon
+	title := gettext.Tr("Login Reminder")
+	actions := []string{"details", gettext.Tr("Details")}
+	notifyId, err := notifi.Notify(0, "dde-control-center", 0, "preferences-system", title, body, actions, nil, 0)
+	if err != nil {
+		logger.Warningf("failed to send notify: %s", err)
+		return
+	}
+
+	_, err = notifi.ConnectActionInvoked(func(id uint32, actionKey string) {
+		if id != notifyId || actionKey != "details" {
+			return
+		}
+
+		lastLoginTime, err := time.Parse("2006-01-02 15:04:05.999999999 -0700 MST", res.LastLogin.Time)
+		if err != nil {
+			logger.Warning("wrong time format:", err)
+		}
+		lastLoginTimeStr := lastLoginTime.Format(loginReminderTimeFormat)
+
+		content := fmt.Sprintf("<p>%s</p>", res.Username)
+		content += fmt.Sprintf("<p>%s</p>", address)
+		content += "<p>" + fmt.Sprintf(gettext.Tr("Login time: %s"), currentLoginTimeStr) + "</p>"
+		content += "<p>" + fmt.Sprintf(gettext.Tr("Last login: %s"), lastLoginTimeStr) + "</p>"
+		content += "<p><b>" + fmt.Sprintf(gettext.Tr("Your password will expire in %d days"), daysLeft) + "</b></p>"
+		content += "<br>"
+		content += "<p>" + fmt.Sprintf(gettext.Tr("%d login failures since the last successful login"), res.FailCountSinceLastLogin) + "</p>"
+
+		cmd := exec.Command(cmdDdeHintsDialog, title, content)
+		err = cmd.Start()
+		if err != nil {
+			logger.Warning("failed to start dde-hints-dialog:", err)
+			return
+		}
+
+		go func() {
+			err = cmd.Wait()
+			if err != nil {
+				logger.Warning("failed to run dde-hints-dialog", err)
+				return
+			}
+		}()
+	})
+	if err != nil {
+		logger.Warning("connect to ActionInvoked failed:", err)
+	}
+
+	// 通知不显示在通知中心面板，故在时间到了后，关闭通知
+	time.AfterFunc(loginReminderTimeout, func() {
+		notifi.CloseNotification(0, notifyId)
+
+		notifi.RemoveAllHandlers()
+
+		sigLoop.Stop()
+	})
+}
+
+func getFirstIPAddress() string {
+	ifaces, err := net.Interfaces()
+	if err != nil {
+		panic(err)
+	}
+
+	for _, iface := range ifaces {
+		if iface.Name == "lo" {
+			continue
+		}
+
+		addrs, err := iface.Addrs()
+		if err != nil {
+			logger.Warningf("failed to get address of %s: %s", iface.Name, err)
+			continue
+		}
+
+		for _, addr := range addrs {
+			// remove the netmask
+			return strings.Split(addr.String(), "/")[0]
+		}
+	}
+
+	return "127.0.0.1"
+}
