@@ -2,11 +2,11 @@ package display
 
 import (
 	"github.com/godbus/dbus"
-	"pkg.deepin.io/lib/dbusutil"
-	"pkg.deepin.io/lib/log"
+	"github.com/linuxdeepin/go-lib/dbusutil"
+	"github.com/linuxdeepin/go-lib/log"
 )
 
-var logger = log.NewLogger("daemon/display")
+var logger = log.NewLogger("daemon/wl_display")
 
 const (
 	dbusServiceName = "com.deepin.daemon.Display"
